﻿using System;
using System.Collections.Generic;
using System.Linq;

namespace Advanced
{
    class Program
    {
        delegate void DelExample(string message);

        static DelExample del1 = new DelExample(MethodPrint);
        // or 
        static DelExample del2 = MethodPrint;
        // or anonymous using lambda expression 
        static DelExample del3 = (string msg) => Console.WriteLine(msg);

        // target method
        static void MethodPrint(string message)
        {
            Console.WriteLine(message);
        }

        static void MethodPrint2(string message)
        {
            Console.WriteLine("This is the message: " + message);
        }


        static int Lenght(string str)
        {
            return str.Length;
        }

        static string Join(string str1, string str2)
        {
            return str1 + str2;
        }

        static void PrintVertically(Vector<int> v)
        {
            Console.WriteLine($"[{v.X}]");
            Console.WriteLine($"[{v.Y}]");
            Console.WriteLine($"[{v.Z}]");
        }

        static void Main(string[] args)
        {
            //delegates
            Console.WriteLine("---------- DELEGATES ----------------");
            del1.Invoke("Delegate 1 says - Hello World!");
            del2("Delegate 2 says - Hello World!");
            del3("Delegate 3 says - Hello World!");
            DelExample del4 = MethodPrint2;
            del4("Delegate 4 says - Hello World!");
            //delegates can be combined
            del1 += del2 + del3 + del4;
            del1("All together");
            del1 -= del4;
            del1("Almost all");

            //Generic Types
            Console.WriteLine("---------- GENERIC ----------------");
            Vector<int> iVector = new Vector<int>(1, 2, 3);
            Vector<double> dVector = new Vector<double>(2.4, 1.2, 4.5);
            iVector.Print();
            dVector.Print();

            //Function example
            Console.WriteLine("---------- FUNCTIONS ----------------");
            Func<string, int> len = Lenght;
            int l = len("Hello World");
            Console.WriteLine($"lenght is: {l}");

            Func<string, string, string> join = Join;
            string strJoined = join("Hello ", "World!");
            Console.WriteLine(strJoined);

            //Action example
            Console.WriteLine("---------- ACTIONS ----------------");
            Vector<int> iVec = new Vector<int>(1, 2, 3);
            //the simplest
            Action<Vector<int>> print1 = PrintVertically;
            //new
            Action<Vector<int>> print2 = new Action<Vector<int>>(PrintVertically);
            //anonymous
            Action<Vector<int>> print3 = delegate (Vector<int> v)
            {
                Console.WriteLine($"[{v.X}]");
                Console.WriteLine($"[{v.Y}]");
                Console.WriteLine($"[{v.Z}]");
            };
            //lambda
            Action<Vector<int>> print4 = v =>
            {
                Console.WriteLine($"[{v.X}]");
                Console.WriteLine($"[{v.Y}]");
                Console.WriteLine($"[{v.Z}]");
            };
            print1(iVec);
            print2(iVec);
            print3(iVec);
            print4(iVec);

            //Predicate example
            Console.WriteLine("---------- PREDICATES ----------------");
            List<Vector<int>> vList = new List<Vector<int>>();
            Random rand = new Random();
            for (int i = 0; i < 10; i++)
            {
                vList.Add(new Vector<int>(rand.Next(20), rand.Next(20), rand.Next(20)));
            }
            foreach (var v in vList)
            {
                v.Print();
            }

            var smallVectors = vList.Where(v => v.X < 10 && v.Y < 10 && v.Z < 10);
            Console.WriteLine($"I found {smallVectors.Count()} small vectors");
            foreach (var v in smallVectors)
            {
                v.Print();
            }

            //extension method
            Console.WriteLine("---------- EXTENSIONS ----------------");
            string s1 = "aa";
            string s2 = "";
            string s3 = null;
            Console.WriteLine("s1 is null or empty: " + s1.IsNullOrEmpty());
            Console.WriteLine("s2 is null or empty: " + s2.IsNullOrEmpty());
            Console.WriteLine("s3 is null or empty: " + s3.IsNullOrEmpty());

            string s4 = null;
            //if (s4!=null && s4.Length > 0) Console.WriteLine("s4 length > 0");
            if (!s4.IsNullOrEmpty()) Console.WriteLine("s4 length > 0");
        }
    }
}
