﻿using System;
using System.Collections.Generic;
using System.IO;

namespace Basic
{
    class Program
    {

        static void DateTimeExamples()
        {
            //assigns default value 01/01/0001 00:00:00
            DateTime dt1 = new DateTime();
            //assigns year, month, day
            DateTime dt2 = new DateTime(2020, 10, 08);
            //assigns year, month, day, hour, min, seconds
            DateTime dt3 = new DateTime(2020, 10, 08, 19, 10, 20);
            //assigns year, month, day, hour, min, seconds, UTC timezone
            DateTime dt4 = new DateTime(2020, 10, 08, 19, 10, 20, DateTimeKind.Utc);

            long currentTicks = DateTime.Now.Ticks; //Ticks are 100 - nanosecond intervals that have elapsed since January 1, 0001, at 00:00:00.000 in the Gregorian calendar.
            DateTime dt = new DateTime(currentTicks);
            long minticks = DateTime.MinValue.Ticks;
            long maxticks = DateTime.MaxValue.Ticks;
            Console.WriteLine($"current ticks {currentTicks} current Date: {dt} min ticks: {minticks} max ticks: {maxticks}");

            DateTime dt5 = new DateTime(2020, 12, 31);
            DateTime dt6 = new DateTime(2020, 12, 30,12,12,12);
            TimeSpan ts = new TimeSpan(25, 20, 55);
            DateTime newDate = dt5.Add(ts);
            Console.WriteLine(newDate);
            TimeSpan ts2 = dt5-dt6;
            Console.WriteLine(ts2);

            Console.WriteLine("Date String Current Culture: " + dt.ToString("d"));
            Console.WriteLine("MM/dd/yyyy Format: " + dt.ToString("MM/dd/yyyy"));
            Console.WriteLine("dddd, dd MMMM yyyy Format: " + dt.ToString("dddd, dd MMMM yyyy"));
            Console.WriteLine("MM/dd/yyyy h:mm tt Format: " + dt.ToString("MM/dd/yyyy h:mm tt"));
            Console.WriteLine("MMMM dd Format:" + dt.ToString("MMMM dd"));
            Console.WriteLine("HH:mm:ss Format: " + dt.ToString("HH:mm:ss"));
            Console.WriteLine("hh:mm tt Format: " + dt.ToString("hh:mm tt"));
            Console.WriteLine("Short Date String: " + dt.ToShortDateString());
            Console.WriteLine("Long Date String: " + dt.ToLongDateString());
            Console.WriteLine("Short Time String: " + dt.ToShortTimeString());
            Console.WriteLine("Long Time String: " + dt.ToLongTimeString());

            var str = "2020-12-23 14:35:00";
            DateTime dts;
           
            var success = DateTime.TryParse(str, out dts);
            if (success)
                Console.WriteLine($"Date from string: {dts}");
            else
                Console.WriteLine($"{str} is not a valid date string");
        }

        static void PrintRanges()
        {
            
            Console.WriteLine($"sbyte {sbyte.MinValue}..{sbyte.MaxValue}");
            Console.WriteLine($"byte {byte.MinValue}..{byte.MaxValue}");
            Console.WriteLine($"short {short.MinValue}..{short.MaxValue}");
            Console.WriteLine($"ushort {ushort.MinValue}..{ushort.MaxValue}");
            Console.WriteLine($"int {int.MinValue}..{int.MaxValue}");
            Console.WriteLine($"uint {uint.MinValue}..{uint.MaxValue}");
            Console.WriteLine($"long {long.MinValue}..{long.MaxValue}");
            Console.WriteLine($"ulong {ulong.MinValue}..{ulong.MaxValue}");
            Console.WriteLine($"float {float.MinValue}..{float.MaxValue}");
            Console.WriteLine($"double {double.MinValue}..{double.MaxValue}");
            Console.WriteLine($"decimal {decimal.MinValue}..{decimal.MaxValue}");
        }

        static void StringExamples()
        {
            string c = "S";
            string w = "Word";
            string t = "This is a sentence.";
            
            //string as array
            char[] chars = { 's', 't', 'r', 'i', 'n', 'g' };
            string str = new string(chars);
            foreach (char chr in str)
            {
                Console.WriteLine(chr);
            }
            Console.WriteLine(str);
            string text1 = "This is a \"string\"";
            string text2 = @"c:\tmp\dir";
            string text3 = "c:\\tmp\\dir2";
            Console.WriteLine(text1);
            Console.WriteLine(text2);
            Console.WriteLine(text3);
        }

        static void IplicitlyTypedExamples()
        {

            var isOk = true;
            Console.WriteLine("Type of isOk is {0}", isOk.GetType());

            var str = "String sample";
            Console.WriteLine("Type of str is {0}", str.GetType());

            var n = 5;
            Console.WriteLine("Type of n is {0}", n.GetType());

            var dbl = 100.50d;
            Console.WriteLine("Type of dbl is {0}", dbl.GetType());

            
            var ano = new { name = "Steve" };
            Console.WriteLine("Type of ano is {0}", ano.GetType());

            var arr = new[] { 1, 10, 20, 30 };
            Console.WriteLine("Type of arr is {0}", arr.GetType());

            var file = new FileInfo("MyFile");
            Console.WriteLine("Type of file is {0}", file.GetType());
        }

        static void AnonymousExample()
        {
            var car = new { Id = 1, Brand = "Mazda", Model = "6", Year = 2019 };
            Console.WriteLine($"car id: { car.Id} Brand: { car.Brand} Model: { car.Model} Year: { car.Year}");
            // car.Year = 2020; 
            // anonymous types are read only
        }

        static void DynamicExample()
        {
            dynamic myVariable = 1;
            Console.WriteLine($"MyVariable: {myVariable}");
            myVariable = "Other type";
            Console.WriteLine($"MyVariable: {myVariable}");
            myVariable = DateTime.Now;
            Console.WriteLine($"MyVariable: {myVariable}");
        }

        static void IfElseExample()
        {
            int x = 5, y = 10;

            if (x < y)
            {
                Console.WriteLine("x is less than y");
            }

            if (x > y)
            {
                Console.WriteLine("x is greater than y");
            }
            else
            {
                Console.WriteLine("x is less or equal to y");
            }
            //Ternary Operator ?:
            var greater = x > y ? x : y;
            Console.WriteLine($"Greater number is {greater}");
        }

        static void SwitchExample()
        {
            Console.Write("Give number from 1 to 3: ");
            int i = int.Parse(Console.ReadLine());

            switch (i)
            {
                case 1:
                    Console.WriteLine($"You have choose one");
                    break;
                case 2:
                    Console.WriteLine($"You have choose two");
                    break;
                case 3:
                    Console.WriteLine($"You have choose three");
                    break;
                default:
                    Console.WriteLine($"You have choose wrong number {i}");
                    break;
            }
        }

        static void ArraysAndListsExample()
        {
            int[] numbers = new int[5] { 2, 4, 6, 8, 10 };
            string[] colors = new string[3] { "red", "green", "blue" };
            Console.WriteLine($"numbers[0] = {numbers[0]}");
            Console.WriteLine($"numbers[1] = {numbers[1]}");
            Console.WriteLine($"colors[0] = {colors[0]}");
            Console.WriteLine($"colors[1] = {colors[1]}");
            colors[0] = "black";
            Console.WriteLine($"colors[0] = {colors[0]}");
            List<string> names = new List<string>();
            names.Add("John");
            names.Add("Tom");
            names.Add("Anna");
            Console.WriteLine($"{names[0]} {names[1]} {names[2]}");
            
        }

        static void ForeachExample()
        {
            string[] colors = new string[3] { "red", "green", "blue" };
            List<string> names = new List<string>();
            names.Add("John");
            names.Add("Tom");
            names.Add("Anna");
            for (int i = 0; i < colors.Length; i++)
            {
                Console.WriteLine(colors[i]);
            }
            foreach (var item in names)
            {
                Console.WriteLine(item);
            }
        }

        static void MultiDimensionalArraysExample()
        {
            int[,] array2d; // 2D array
            int[,,] array3d; // 3D array
            array2d = new int[3, 2]{
                                   {1, 2},
                                   {3, 4},
                                   {5, 6}
            };
            for (int i = 0; i < 3; i++)
            {
                Console.WriteLine($"{ array2d[i, 0]} { array2d[i, 1]}");
            }
            array3d = new int[2, 2, 2]{
                                      { {1, 2}, {3, 4} },
                                      { {5, 6}, {7, 8} }
            };
            for (int i = 0; i < 2; i++)
            {
                for (int j = 0; j < 2; j++)
                {
                    Console.WriteLine($"{ array3d[i, j, 0]} { array3d[i, j, 1]}");
                }        
            }
        }


        static void JaggedArraysExample()
        {
            int[][] array1 = new int[2][]; //array of two single-dimensional arrays 
            int[][,] array2 = new int[3][,]; // array of three two-dimensional arrays 
            
            array1[0] = new int[4] { 1, 2, 3, 4 };
            array1[1] = new int[5] { 5, 6, 7, 8, 9 };
            foreach (int[] arr in array1)
            {
                foreach (var item in arr)
                {
                    Console.WriteLine( item );
                }
            }
            for (int i = 0; i < 2; i++)
            {
                for (int j = 0; j < array1[i].Length; j++)
                {
                    Console.WriteLine($"array1[{i}][{j}] = {array1[i][j]}");
                }
            }
            int[][][] arrayOfarraysOfarrays = new int[2][][] 
                            {
                                new int[2][]
                                {
                                    new int[3] { 1, 2, 3},
                                    new int[2] { 4, 5}
                                },
                                new int[1][]
                                {
                                    new int[3] { 7, 8, 9}
                                }
                            };
            Console.WriteLine(arrayOfarraysOfarrays[0][0][0]); //1
            Console.WriteLine(arrayOfarraysOfarrays[1][0][1]); //8
        }


        static void WhileExample()
        {
            int x = 0, y = 1;
            while (x < 3)
            {
                x++;
                while (y < 5)
                {
                    Console.WriteLine($"x = {x}  y = {y}");
                    y++;
                }
                y = 1;
            }
            int i = 0;
            do
            {
                Console.WriteLine(++i);
            } while (i < 5);
        }

        static void StructureExample()
        {
            Person person = new Person();
            person.Age = 18;
            person.Name = "Johny";
            person.Surname = "Walker";
            Console.WriteLine($"{person.Name} {person.Surname} {person.Age}");
            Person person2 = new Person("Jack", "Daniels", 21);
            Console.WriteLine($"{person2.Name} {person2.Surname} {person2.Age}");
            Person person3 = new Person();
            person3.SetPerson("Captain", "Morgan", 25);
            Console.WriteLine($"{person3.Name} {person3.Surname} {person3.Age}");
            Person person4 = new Person("Borys");
            Console.WriteLine($"{person4.Name} {person4.Surname} {person4.Age}");

        }

        static void EnumExample()
        {
            Console.WriteLine("Choose level from 0 to 4");
            Level level = (Level)int.Parse(Console.ReadLine());

            switch (level)
            {
                case Level.VeryLow:
                    Console.WriteLine("You have choosen Very Low level");
                    break;
                case Level.Low:
                    Console.WriteLine("You have choosen Low level");
                    break;
                case Level.Normal:
                    Console.WriteLine("You have choosen Normal level");
                    break;
                case Level.High:
                    Console.WriteLine("You have choosen High level");
                    break;
                case Level.VeryHigh:
                    Console.WriteLine("You have choosen Very High level");
                    break;
                default:
                    Console.WriteLine("You have choosen wrong level");
                    break;
            }
        }

        static void VectorExample()
        {
            Vector vec1 = new Vector();
            vec1.X = 1.0;
            vec1.Y = 2.0;
            vec1.Z = -2.5;

            Vector vec2 = new Vector() { X = -0.5, Y = 0, Z = 3.0 };
            Vector vec3 = new Vector(4.2, 2.2, 3.1);
            vec1.Print();
            vec2.Print();
            vec3.Print();
        }

        static void ExceptionsExample()
        {
            try
            {
                //int[] array = new int[2];
                //array[3] = 11; // IndexOutOfRangeException

                int a = int.Parse("a"); //FormatException

                //int zero = 0;   
                //var b = 2 / zero;

                throw new ApplicationException("OMG! We have exception!!!");

            }
            catch (DivideByZeroException zeroEx)
            {
                Console.WriteLine($"type: {zeroEx.GetType()} message: {zeroEx.Message}");
            }
            catch (IndexOutOfRangeException iex)
            {
                Console.WriteLine($"type: {iex.GetType()} message: {iex.Message}");
            }
            catch (Exception ex)
            {
                Console.WriteLine($"type: {ex.GetType()} message: {ex.Message}");
            }
            finally
            {
                Console.WriteLine("Finally block");
            }
        }

        
        


        static void Main(string[] args)
        {

            
            //PrintRanges();
            //StringExamples();
            //DateTimeExamples();
            //IplicitlyTypedExamples();
            //AnonymousExample();
            //DynamicExample();
            //IfElseExample();
            //SwitchExample();
            //ArraysAndListsExample();
            //ForeachExample();
            //MultiDimensionalArraysExample();
            //JaggedArraysExample();
            //WhileExample();
            //StructureExample();
            //EnumExample();
            //VectorExample();
            //ExceptionsExample();
                       
        }
    }
}
